/*-----------------------------------------------------------------------------
 *  FILE: dma_scu1.c
 *
 *      Copyright(c) 1994 SEGA.
 *
 *  PURPOSE:
 *
 *      SCU DMAB
 *
 *  DESCRIPTION:
 *
 *      SCU DMA]@\B
 *
 *  INTERFACE:
 *
 *      < FUNCTIONS LIST >
 *          DMA_ScuInit             -   DMA
 *          DMA_ScuMemCopy          -   DMAf[^]
 *          DMA_ScuResult           -   DMAf[^]I`FbN
 *
 *  CAVEATS:
 *
 *
 *  AUTHOR(S)
 *
 *      1994-08-31  N.T Ver.1.01
 *
 *  MOD HISTORY:
 *
 *      1994-10-08  N.T Ver.1.05
 *
 *-----------------------------------------------------------------------------
 */

/*
 * C STANDARD LIBRARY FUNCTIONS/MACROS DEFINES
 */

/*
 * C VIRTUAL TYPES DEFINITIONS
 */
#include <machine.h>
#include "sega_xpt.h"
#include "sega_csh.h"
#include "sega_int.h"
/*
 * USER SUPPLIED INCLUDE FILES
 */
#include "dma_xpt.h"
#include "dma_scu0.h"
#include "dma_scu1.h"

/*
 * GLOBAL DECLARATIONS
 */

/*
 * LOCAL DEFINES/MACROS
 */
/* SCU Address */
#define CADR_B_BUS_START   ((Uint32)0x5a00000)
#define CADR_B_BUS_END     ((Uint32)0x5fe0000)
#define ADR_B_BUS_START    ((Uint32)0x25a00000)
#define ADR_B_BUS_END      ((Uint32)0x25fe0000)

#define CADR_WORKRAM_L_START   ((volatile void *)0x200000)
#define CADR_WORKRAM_L_END     ((volatile void *)0x300000)
#define ADR_WORKRAM_L_START    ((volatile void *)0x20200000)
#define ADR_WORKRAM_L_END      ((volatile void *)0x20300000)

/*
 * STATIC DECLARATIONS
 */
static void *dma_scu_dis_adr;
static Uint32 dma_scu_cnt;
static Uint8 dma_flg;                   /* DMA̎gpLtO */
static Uint8 dma_start_flg;             /* DMÅJnLtO */

/*
 * STATIC FUNCTION PROTOTYPE DECLARATIONS
 */


/******************************************************************************
 *
 * NAME:    DMA_ScuInit()       -   DMA
 *
 * PARAMETERS :
 *      ȂB
 *
 * DESCRIPTION:
 *      SCȔ܂B
 *
 * PRECONDITIONS:
 *      ȂB
 *
 * POSTCONDITIONS:
 *      ȂB
 *
 * CAVEATS:
 *      ȂB
 *
 ******************************************************************************
 */

void DMA_ScuInit(void)
{
    Uint32 msk;

    msk = get_imask();
    set_imask(15);

    INT_ChgMsk(INT_MSK_DMA0, INT_MSK_NULL);     /* DMA0荞ݏI}XN*/
    dma_start_flg = OFF;                        /* DMA̓X^[gĂȂ   */

    set_imask(msk);                                         /* 荞POP   */
}

/******************************************************************************
 *
 * NAME:    DMA_ScuMemCopy()    -   DMAf[^]
 *
 * PARAMETERS :
 *      (1) void *dst   -   <i>   fBXeBl[VAhX.
 *      (2) void *src   -   <i>   \[XAhX.
 *      (3) Uint32 cnt  -   <i>   ]oCg.
 *
 * DESCRIPTION:
 *      SCUDMA[h1̒ڃ[hɂDMA]܂B
 *
 * PRECONDITIONS:
 *      ȂB
 *
 * POSTCONDITIONS:
 *      ȂB
 *
 * CAVEATS:
 *      ȂB
 *
 ******************************************************************************
 */

void DMA_ScuMemCopy(void *dst, void *src, Uint32 cnt)
{
    DmaScuPrm prm;
    Uint32 msk;

    msk = get_imask();
    set_imask(15);

    if(((dst >= ADR_WORKRAM_L_START) && (dst < ADR_WORKRAM_L_END)) ||
       ((dst >= CADR_WORKRAM_L_START) && (dst < CADR_WORKRAM_L_END)) ||
       ((src >= ADR_WORKRAM_L_START) && (src < ADR_WORKRAM_L_END)) ||
       ((src >= CADR_WORKRAM_L_START) && (src < CADR_WORKRAM_L_END))){
    /* NO DMA ****************************************************************/
        dma_flg = OFF;
        memcpy(dst, src, cnt);
    /* DMA *******************************************************************/
    }else{
        dma_flg = ON;

    dma_scu_dis_adr = dst;
    dma_scu_cnt = (Uint32)cnt;

    prm.dxr = (Uint32)src;
    prm.dxw = (Uint32)dst;
    prm.dxc = cnt;
    prm.dxad_r = DMA_SCU_R4;

    if(((prm.dxw >= ADR_B_BUS_START) & (prm.dxw < ADR_B_BUS_END)) ||
       ((prm.dxw >= CADR_B_BUS_START) & (prm.dxw < CADR_B_BUS_END))){
       prm.dxad_w = DMA_SCU_W2;
    }else{
       prm.dxad_w = DMA_SCU_W4;
    }

    prm.dxmod = DMA_SCU_DIR;
    prm.dxrup = DMA_SCU_KEEP;
    prm.dxwup = DMA_SCU_KEEP;
    prm.dxft = DMA_SCU_F_DMA;
    prm.msk = DMA_SCU_M_DXR    |
              DMA_SCU_M_DXW    ;

    DMA_ScuSetPrm(&prm, DMA_SCU_CH0);
    DMA_ScuStart(DMA_SCU_CH0);
    }
    dma_start_flg = ON;                         /* DMA̓X^[gĂ     */
    set_imask(msk);                                         /* 荞POP   */
}

/******************************************************************************
 *
 * NAME:    DMA_ScuResult()     -   DMAf[^]I`FbN
 *
 * PARAMETERS :
 *      ȂB
 *
 * DESCRIPTION:
 *      SCUDMA[h2̒ڃ[hɂDMA]܂B
 *
 * PRECONDITIONS:
 *      ȂB
 *
 * POSTCONDITIONS:
 *      Uint32 DMA_ScuMemCopy()̌ʁB
 *
 * CAVEATS:
 *      ȂB
 *
 ******************************************************************************
 */

Uint32 DMA_ScuResult(void)
{
    DmaScuStatus status;
    Uint32 msk;

    msk = get_imask();
    set_imask(15);

    if(dma_start_flg == ON){                    /* DMAJnĂ鎞       */
        if(dma_flg == OFF){
            set_imask(msk);                                 /* 荞POP   */
            return(DMA_SCU_END);
        }else{
        DMA_ScuGetStatus(&status, DMA_SCU_CH0);
            if(status.dxmv == DMA_SCU_MV){
                set_imask(msk);                             /* 荞POP   */
                return(DMA_SCU_BUSY);
            }
            CSH_Purge(dma_scu_dis_adr, dma_scu_cnt);
            set_imask(msk);                                 /* 荞POP   */
            return(DMA_SCU_END);
        }
    }else{                                      /* DMAJnĂȂ     */
        set_imask(msk);                                     /* 荞POP   */
        return(DMA_SCU_END);
    }
}

