/*----------------------------------------------------------------------------
 *  dsp_ctrl.c -- DSP Cu CTRL W[
 *  Copyright(c) 1994 SEGA
 *  Written by H.E on 1994-04-04 Ver.0.80
 *  Updated by H.E on 1994-07-25 Ver.1.00
 *
 *  ̃Cu͂cro䏈W[ŁAȉ̃[`܂ށB
 *
 *  DSP_LoadProgram         -  vO[h
 *  DSP_WriteData           -  f[^Cg
 *  DSP_ReadData            -  f[^[h
 *  DSP_Start               -  sJn
 *  SPR_Stop                -  s~
 *  SPR_CheckEnd            -  sI`FbN
 *
 *  ̃Cugpɂ͎̃CN[ht@C`KvB
 *
 *  #include "sega_dsp.h"
 *
 *----------------------------------------------------------------------------
 */

/*
 * USER SUPPLIED INCLUDE FILES
 */
#include "sega_int.h"
#include "sega_dsp.h"
#ifdef _SH
#include <machine.h>
#endif


/*****************************************************************************
 *
 * NAME:  DSP_LoadProgram()    - Load DSP Program
 *
 * PARAMETERS :
 *
 *     (1) Uint8   dst         - <i> crovOq`l̃AhX
 *     (2) Uint32  *src        - <i> [Nq`lvOGAAhX
 *     (3) Uint16  count       - <i> vOTCYiO[hPʁj
 *
 * DESCRIPTION:
 *
 *     cro~Acro֎wvO[hB
 *
 * POSTCONDITIONS:
 *
 *     No exist.
 *
 * CAVEATS:
 *
 *
 *****************************************************************************
 */
void DSP_LoadProgram(Uint8 dst, Uint32 *src, Uint16 count)
{
    Uint32  ctrl;
    int     i;

    /** BEGIN ***************************************************************/
    INT_ChgMsk(INT_MSK_NULL, INT_MSK_DSP);   /* DSP ̊荞݂}XN  */
    DSP_WRITE_REG(DSP_RW_CTRL, 0);    /* DSP Stop                           */
    ctrl = 0x8000 | dst;
    DSP_WRITE_REG(DSP_RW_CTRL, ctrl); /* Set Program Pos                    */
    for(i=0; i<count; i++)
        DSP_WRITE_REG(DSP_W_PDAT, *src++); /* Write DSP Program             */
}


/*****************************************************************************
 *
 * NAME:  DSP_WriteData()      - Write Data in the DSP Data RAM
 *
 * PARAMETERS :
 *
 *     (1) Uint8   dst         - <i> crof[^q`l̃AhX
 *     (2) Uint32  *src        - <i> [Nq`lf[^GAAhX
 *     (3) Uint16  count       - <i> f[^TCYiO[hPʁj
 *
 * DESCRIPTION:
 *
 *     cro~Acrõf[^q`l֎wf[^ށB
 *
 * POSTCONDITIONS:
 *
 *     No exist.
 *
 * CAVEATS:
 *
 *
 *****************************************************************************
 */
void DSP_WriteData(Uint8 dst, Uint32 *src, Uint16 count)
{
    Uint32  ramAddr;
    int     i;

    /** BEGIN ***************************************************************/
    DSP_WRITE_REG(DSP_RW_CTRL, 0);      /* DSP Stop                         */
    ramAddr = dst;
    DSP_WRITE_REG(DSP_W_DADR, ramAddr); /* Write Data Address               */
    for(i=0; i<count; i++)
        DSP_WRITE_REG(DSP_RW_DDAT, *src++);   /* Write Data                 */
}


/*****************************************************************************
 *
 * NAME:  DSP_ReadData()       - Read Data from the DSP Data RAM
 *
 * PARAMETERS :
 *
 *     (1) Uint32  *dst        - <o> [Nq`lf[^GAAhX
 *     (2) Uint8   src         - <i> crof[^q`l̃AhX
 *     (3) Uint16  count       - <i> f[^TCYiO[hPʁj
 *
 * DESCRIPTION:
 *
 *     cro~Acrõf[^q`lwf[^ǂݏoB
 *
 * POSTCONDITIONS:
 *
 *     No exist.
 *
 * CAVEATS:
 *
 *
 *****************************************************************************
 */
void DSP_ReadData(Uint32 *dst, Uint8 src, Uint16 count)
{
    Uint32  ramAddr;
    int     i;

    /** BEGIN ***************************************************************/
    DSP_WRITE_REG(DSP_RW_CTRL, 0);        /* DSP Stop                       */
    ramAddr = src;
    DSP_WRITE_REG(DSP_W_DADR, ramAddr++); /* Write Data Address             */
    for(i=0; i<count; i++)
        *dst++ = DSP_READ_REG(DSP_RW_DDAT);   /* Read  Data                 */
}


/*****************************************************************************
 *
 * NAME:  DSP_Start()          - Execute DSP Program
 *
 * PARAMETERS :
 *
 *     (1) Uint8   pc          - <i> crovO̊Jnʒu
 *
 * DESCRIPTION:
 *
 *     w肳ꂽcrovO̎sJnʒusB
 *
 * POSTCONDITIONS:
 *
 *     No exist.
 *
 * CAVEATS:
 *
 *
 *****************************************************************************
 */
void DSP_Start(Uint8 pc)
{
    Uint32  ctrl;

    /** BEGIN ***************************************************************/
    DSP_WRITE_REG(DSP_RW_CTRL, 0);    /* DSP Stop                           */
    ctrl = 0x18000 | pc;
    DSP_WRITE_REG(DSP_RW_CTRL, ctrl); /* Set Program pc & start DSP         */
}


/*****************************************************************************
 *
 * NAME:  DSP_Stop()           - Stop DSP Program
 *
 * PARAMETERS :
 *
 *     No exist.
 *
 * DESCRIPTION:
 *
 *     ŝcrovO~B
 *
 * POSTCONDITIONS:
 *
 *     No exist.
 *
 * CAVEATS:
 *
 *
 *****************************************************************************
 */
void DSP_Stop(void)
{
    /** BEGIN ***************************************************************/
    DSP_WRITE_REG(DSP_RW_CTRL, 0);    /* DSP Stop                           */
}


/*****************************************************************************
 *
 * NAME:  DSP_CheckEnd()           - Check DSP Process End
 *
 * PARAMETERS :
 *
 *     No exist.
 *
 * DESCRIPTION:
 *
 *     crovȌI`FbNB
 *
 * POSTCONDITIONS:
 *
 *     Uint8   result              - <o> ItO
 *                                       DSP_END     = sI
 *                                       DSP_NOT_END = s
 *
 * CAVEATS:
 *
 *
 *****************************************************************************
 */
Uint8 DSP_CheckEnd(void)
{
#if	0
	/*
	**1995-07-26	q
	**	gĂȂ̂ō폜
	*/
    Uint32  vbr, *irqVector;
    int     iMask;
    Uint32  *l, i;
#endif
    /** BEGIN ***************************************************************/
    if((*(volatile Uint32*)0x25fe00a4) & 0x20) {
        for(; DSP_READ_REG(DSP_RW_CTRL) & 0x800000; ) {}
        *(volatile Uint32*)0x25fe00a4  = ~0x20;
        *(volatile Uint16*)0xfffffe92 |= 0x10;   /* cash parge */ 
        return DSP_END;
    }  else
        return DSP_NOT_END;
#if 0
    if(INT_GetStat() & INT_ST_DSP) {
        for(; DSP_READ_REG(DSP_RW_CTRL) & 0x800000; ) {}
        INT_ResStat(INT_ST_DSP);
        *((volatile Uint16*)0xfffffe92) |= 0x10;   /* cash parge */ 
        return DSP_END;
    }  else
        return DSP_NOT_END;
#endif
}

/*  end of file */
